var _ = require( "lodash" );
var replFn = require( "repl" );
var when = require( "when" );
var rabbot = require( "../src" );

var repl = replFn.start( {
	prompt: '> ',
	ignoreUndefined: true,
	eval: evalCommand
} );

var topologies = {};

var commands;
commands = {
	connect: {
		format: "connect url",
		description: "attempts to connect to a rabbit broker via uri",
		handle: function( uri ) {
			var topology = rabbot.addConnection( { uri: uri } );
			topologies[ topologies.name ] = topology;
			var deferred = when.defer();
			rabbot.once( "connected", () => {
				console.log( "connected" );
				deferred.resolve();
			} )
			rabbot.once( "failed", ( err ) => {
				console.log( "connection attempt failed", err.stack ? err.stack : err );
				deferred.reject();
			} );
			return deferred.promise;
		}
	},
	ex: {
		format: "ex name type",
		description: "create an auto-deleted fanout exchange on the connection",
		handle: function( name, type ) {
			return rabbot.addExchange( name, type, { autoDelete: true } )
				.then( 
					() => console.log( "done" ),
					( err ) => console.log( "error", err )
				);
		}
	},
	b: {
		format: "b exchangeName queueName",
		description: "binds a queue to an exchange",
		handle: function( exchaneName, queueName ) {
			
		}
	},
	q: {
		format: "q name options",
		description: "create an autodeleted queue on the connection",
		handle: function( name, options ) {
			return rabbot.addQueue( name, options )
				.then( () => console.log( "done" ) );
		}
	},
	h: {
		format: "h ",
		description: "create a queue on the connection",
		handle: function( name, options ) {
			return rabbot.addQueue( name, options )
				.then( () => console.log( "done" ) );
		}
	},
	help: {
		handle: function( commandName ) {
			if( commandName  ) {
				var command = commands[ commandName ];
				if( command ) {
					console.log( `${command.format} - ${command.description}` );
				} else {
					console.log( `Command ${commandName} does not exist` );
				}
			} else {
				console.log( "Command list:" );
				_.each( commands, ( command ) => {
					if( command.format ) {
						console.log( `\t${command.format} - ${command.description}` );
					} 
				} );
			}
		}
	},
	
	exit: {
		format: "exit",
		description: "quit to the shell",
		handle: function() {
			process.exit();
		}
	}
};

function evalCommand( cmd, context, filename, callback ) {
	var args = parse( cmd );
	var command = args[ 0 ];
	var definition = global[ command ] || context[ command ];
	if( definition ) {
		console.log( ":", definition );
		callback( null );
	} else if( commands[ command ] ) {
		var result = commands[ command ].handle.apply( context, args.slice( 1 ) );
		if( result && result.then ) {
			function done() {
				callback( null );
			}
			result.then( done, done );
		} else {
			callback( null );
		}
	}
}

function parse( line ) {
	var rgx = /([a-zA-Z0-9_\-:?=.&$%\/]+)/g;
	var args = [];
	for( var m; m = rgx.exec( line ); ) {
		args.push( m[ 0 ] );
	}
	return args;
}